; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 11/2015															   
TWI_INI:
	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 674 Hz = 22.000.000 / (16 + 2 * 255 * 4^3)	

	; TWI TWBR 				Bit RATE Register	
	ldi temp, (TWI_BIT_RATE);0..255				
	STS TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, (TWI_PRESCALER)
	andi temp,0b00000011	; 0...3 zulssig	
	STS TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	STS TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	STS TWCR,temp
	; PULL-UP ein (ca. 30k)						
	sbi (TWI_SCL_PORT),(TWI_SCL_PIN_NR)
	sbi (TWI_SDA_PORT),(TWI_SDA_PIN_NR)
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 11/2015															   
; TWI SCANNER																	
TWI_SCAN:
	clr temp1	; SLAVE Adresse	
TWI_SCAN_S:
	WDR
	push temp1
	; START			
	call TWI_MASTER_TRANSMITTER_START_TEMP1
	pop temp1
	tst TWI_FEHLER_CODE 		;TWI_FEHLER_CODE
	brne TWI_SCAN_w	
	push temp1
	; Anzeige		
	call ANZEIGE_3_STELLEN_TEMP1
	call ASCII_SPACE
	pop temp1
TWI_SCAN_w:
	inc temp1	; SLAVE Adresse	
	cpi temp1,127+1
	brne TWI_SCAN_S
	call TWI_STOP
	STS(adr_ERROR_CODE),NULL
	STS(adr_ERROR_WERT),NULL
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 11/2015															   
; Anzeige, wenn SDA oder SCL gegen GND gezogen wird							   
TWI_BUS_FREE_CHECK:
	; kurze PAUSE, wichtig !
	call WAIT_1ms	
	; SCL			
	sbic (TWI_SCL_PIN),(TWI_SCL_PIN_NR)
	rjmp TWI_BUS_FREE_CHECK_SDA
	call GRAFIC_CLEAR
	call ASCII_CURSOR_ZEILE_1
	call ASCII_S
	call ASCII_C
	call ASCII_L
	call wait_1s
	rjmp TWI_BUS_FREE_CHECK
;------------------------------------------------------------------------------
TWI_BUS_FREE_CHECK_SDA:
	; SDA			
	sbic (TWI_SDA_PIN),(TWI_SDA_PIN_NR)
	ret
	call GRAFIC_CLEAR
	call ASCII_CURSOR_ZEILE_1
	call ASCII_S
	call ASCII_D
	call ASCII_A
	call wait_1s
	call ASCII_CURSOR_ZEILE_1
	rjmp TWI_BUS_FREE_CHECK
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 11/2015															   
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	STS TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 11/2015															   
;																			   
; START-ROUTINE	(MASTER TRANSMITTER)										   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUTPUT: 	TWI_FEHLER_CODE (FEHLER_CODE)									   
;																			   
TWI_MASTER_TRANSMITTER_START_TEMP1:
	WDR
	clr  TWI_FEHLER_CODE ;TWI_FEHLER_CODE
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	STS TWCR, temp	
	rcall TWI_TWINT_WAIT						; warten, bis TWINT wieder = 1	
	tst  TWI_FEHLER_CODE 						;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_START_RUN
	ret									
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_RUN:
	; Check TWSR						
	LDS temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10							; 0x10 (repeated START)
	breq TWI_MASTER_TRANSMITTER_START_w
	; ERROR		
	ldi temp,(ERROR_TWI_MT_START)
	mov TWI_FEHLER_CODE, temp ;TWI_FEHLER_CODE
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1								; 1 x links
	STS TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_WAIT									
	tst  TWI_FEHLER_CODE 										;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_START_w2
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_w2:
	; Check TWSR						
	LDS temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	brne TWI_MASTER_TRANSMITTER_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_ERROR:
	; ERROR		
	ldi temp,(ERROR_TWI_MT_SLA)
	mov TWI_FEHLER_CODE, temp  ;TWI_FEHLER_CODE
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 08/2015															   
; 																			   
; Ein BYTE zum SLAVE zu senden										   	   	   
; 																			   
; INPUT:	temp1 <= DATEN													   
; OUT: 		TWI_FEHLER_CODE (FEHLER_CODE)   								   
; 																			   
TWI_MASTER_TRANSMITTER_DATA_TEMP1:
	; FEHLERPRFUNG						
	WDR
	tst  TWI_FEHLER_CODE 								;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATA_w		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATA_w:
	; 	Load DATA						
	STS TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_WAIT
	tst  TWI_FEHLER_CODE 								;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATA_w2		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATA_w2:
	; Check TWSR						
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_DATA_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATA_ERROR:
	; ERROR								
	ldi temp,(ERROR_TWI_MT_DATA)
	mov TWI_FEHLER_CODE, temp ;TWI_FEHLER_CODE
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 08/2015															   
;																			   
; TWI/I2C START-ROUTINE														   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUT: 		TWI_FEHLER_CODE (FEHLER_CODE)   												   
;																			   
TWI_MASTER_RECEIVER_START_TEMP1:
	; FEHLER lschen					
	WDR
	clr TWI_FEHLER_CODE 			;TWI_FEHLER_CODE
	; SRAM lschen						
	STS	(adr_TWI_BEREICH_READ+0),NULL
	STS	(adr_TWI_BEREICH_READ+1),NULL
	STS	(adr_TWI_BEREICH_READ+2),NULL
	STS	(adr_TWI_BEREICH_READ+3),NULL
	STS	(adr_TWI_BEREICH_READ+4),NULL
	STS	(adr_TWI_BEREICH_READ+5),NULL
	STS	(adr_TWI_BEREICH_READ+6),NULL
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	STS TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_WAIT									
	tst  TWI_FEHLER_CODE 								;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_RECEIVER_START_RUN		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_RUN:
	; Check TWSR 										
	LDS temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RECEIVER_START_w
	rjmp TWI_MASTER_RECEIVER_START_ERROR
TWI_MASTER_RECEIVER_START_w:
	; Load SLA +"R" (R=1)								
	lsl  temp1	; 1 x links
	ori temp1, 0b00000001 
	STS TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_WAIT									
	tst  TWI_FEHLER_CODE 								;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_RECEIVER_START_w2		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_w2:
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_ERROR:
	; ERROR						
	ldi temp,(ERROR_TWI_MR_SLA)
	mov TWI_FEHLER_CODE, temp ;TWI_FEHLER_CODE
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 08/2015															   
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTE empfangen werden sollen											
;																				
; OUTPUT: temp1																	
; OUTPUT: TWI_FEHLER_CODE(FEHLER_CODE)	   													
;																				
TWI_MASTER_RECEIVER_DATA_TEMP1:
	; Fehlerprfung										
	WDR
	tst  TWI_FEHLER_CODE ;TWI_FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATA_w
	ret
TWI_MASTER_RECEIVER_DATA_w:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_WAIT									
	tst  TWI_FEHLER_CODE 								;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_RECEIVER_DATA_w2		; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_w2:
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_DATA_ERROR
	; DATEN einlesen									
	LDS temp1, TWDR	
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_ERROR:
	; ERROR												
	ldi temp,(ERROR_TWI_MR_DATA)
	mov TWI_FEHLER_CODE, temp ;TWI_FEHLER_CODE
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
	clr temp1				
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 08/2015															   
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! BYTE empfangen werden soll, oder wenn								
; das letzte! BYTE empfangen werden soll										
;																				
; OUTPUT: temp1																	
; OUTPUT: TWI_FEHLER_CODE(FEHLER_CODE)	   													
;																				
TWI_MASTER_RECEIVER_DATA_LAST_BYTE_TEMP1:
	; Fehlerprfung										
	tst  TWI_FEHLER_CODE ;TWI_FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w
	ret
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_WAIT									
	tst  TWI_FEHLER_CODE 									;TWI_FEHLER_CODE; Fehler =0 ?	
	breq TWI_MASTER_RECEIVER_DATA_LAST_BYTE_w1	; ja			
	ret											; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_LAST_BYTE_w1:
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_DATA_LAST_BYTE_ERROR	; ERROR
	; DATEN einlesen		
	LDS temp1, TWDR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATA_LAST_BYTE_ERROR:
	; ERROR
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov TWI_FEHLER_CODE, temp ;TWI_FEHLER_CODE	
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
	clr temp1
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 08/2015															   
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	STS TWCR, temp	
	; kurze Pause
	call wait_1us
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 11/2015															   
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
;																			   
; OUT: TWI_FEHLER_CODE														   
; 																			   
TWI_TWINT_WAIT:
	push temp1
	push temp2
	push temp3
	clr temp1
	clr temp2
	ldi temp3,10
TWI_TWINT_WAIT_s:
	WDR
	LDS temp, TWCR		; TWCR auslesen	
	sbrc temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG nicht gesetzt ist
	rjmp TWI_WAIT_FERTIG
	dec temp1
	brne TWI_TWINT_WAIT_s
	dec temp2
	brne TWI_TWINT_WAIT_s
	dec temp3
	brne TWI_TWINT_WAIT_s
	; ERROR					
	ldi temp,(ERROR_TWI_TWINT)
	STS(adr_ERROR_CODE),temp
	STS(adr_ERROR_WERT),NULL
	mov TWI_FEHLER_CODE,temp
;-------------------------------------------------------------------------------
TWI_WAIT_FERTIG:
	pop temp3
	pop temp2
	pop temp1
	ret
; #############################################################################
; #############################################################################
; #############################################################################


